import javax.swing.*;
import javax.swing.filechooser.FileNameExtensionFilter;
import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Optional;
import org.apache.logging.log4j.Logger;

public class FormMapWithSetWarships extends JFrame{

    private JComboBox СomboBoxSelectorMap;
    private JButton ButtonAddWarship;
    private JTextField TextBoxPosition;
    private JButton ButtonRemoveWarship;
    private JButton ButtonShowStorage;
    private JButton ButtonShowOnMap;
    private JButton ButtonDown;
    private JButton ButtonUp;
    private JButton ButtonLeft;
    private JButton ButtonRight;
    private JPanel PictureBox;
    private JPanel MainPanel;
    private JPanel GroupBoxTools;
    private JTextField TextFieldMap;
    private JButton CreateMapButton;
    private JButton DeleteMapButton;
    private JList ListBoxMaps;
    private JPanel MapPanel;
    private JButton ButtonShowDeleted;
    private JMenuBar MenuBar;
    private Image bufferedImage;
    private MapsCollection _mapsCollection;
    private final HashMap<String,AbstractMap> _mapsDict=new HashMap<String,AbstractMap>(){
        {
            put("Первая карта",new SimpleMap());
            put("Вторая карта",new SecondMap());
        }
    };
    private Logger logger;

    public FormMapWithSetWarships(Logger logger){
        this();
        this.logger=logger;
    }

    public FormMapWithSetWarships(){
        setTitle("Военный корабль");
        setContentPane(MainPanel);
        setResizable(false);
        setSize(1150,690);
        ShowWindow();
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setVisible(true);
        _mapsCollection=new MapsCollection(getWidth(),getHeight());
        СomboBoxSelectorMap.removeAllItems();
        for(String elem:_mapsDict.keySet()){
            СomboBoxSelectorMap.addItem(elem);
        }
    }

    @Override
    public void paint(Graphics g) {
        super.paint(g);

        if (bufferedImage != null) {
            PictureBox.paintComponents(bufferedImage.getGraphics());
            PictureBox.getGraphics().drawImage(bufferedImage, 0, 0, null);
        }
    }

    private void ReloadMaps()
    {
        int index = ListBoxMaps.getSelectedIndex();
        ListBoxMaps.setListData(_mapsCollection.Keys().toArray(new String[0]));

        if (ListBoxMaps.getModel().getSize() > 0 && (index == -1 || index >= ListBoxMaps.getModel().getSize()))
        {
            ListBoxMaps.setSelectedIndex(0);
        }
        else if (ListBoxMaps.getModel().getSize() > 0 && index > -1 && index < ListBoxMaps.getModel().getSize())
        {
            ListBoxMaps.setSelectedIndex(index);
        }
        repaint();
    }

    private void ShowWindow(){

        MenuBar=new JMenuBar();

        JMenu fileMenu =new JMenu("Файл");
        MenuBar.add(fileMenu);

        JMenuItem SaveMenuItem = new JMenuItem("Сохранить");
        SaveMenuItem.addActionListener(e -> {
            JFileChooser dialog = new JFileChooser();
            dialog.setFileFilter(new FileNameExtensionFilter("txt", "txt"));
            dialog.showSaveDialog(this);

            try {
                _mapsCollection.SaveData(dialog.getSelectedFile().getAbsolutePath());
                logger.info("Сохранение всех карт в файл: "+dialog.getSelectedFile().getAbsolutePath());
                JOptionPane.showMessageDialog(this, "Сохранение всех карт прошло успешно", "Итог", JOptionPane.INFORMATION_MESSAGE);
            } catch (Exception ex) {
                logger.error("Ошибка сохранения всех карт в файл: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Ошибка сохранения всех карт: "+ex.getMessage(), "Итог", JOptionPane.ERROR_MESSAGE);
            }
        });
        fileMenu.add(SaveMenuItem);

        JMenuItem LoadMenuItem = new JMenuItem("Загрузить");
        LoadMenuItem.addActionListener(e -> {
            JFileChooser dialog = new JFileChooser();
            dialog.setFileFilter(new FileNameExtensionFilter("txt", "txt"));
            dialog.showOpenDialog(this);

            try {
                _mapsCollection.LoadData(dialog.getSelectedFile().getAbsolutePath());
                ReloadMaps();
                logger.info("Загрузка всех карт из файла: "+dialog.getSelectedFile().getAbsolutePath());
                JOptionPane.showMessageDialog(this, "Загрузка всех карт прошла успешно", "Итог", JOptionPane.INFORMATION_MESSAGE);
            } catch (Exception ex) {
                logger.error("Ошибка загрузки всех карт из файла: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Ошибка загрузки всех карт"+ex.getMessage(), "Итог", JOptionPane.ERROR_MESSAGE);
            }
        });
        fileMenu.add(LoadMenuItem);

        JMenuItem SaveMapMenuItem = new JMenuItem("Сохранить карту");
        SaveMapMenuItem.addActionListener(e -> {
            JFileChooser dialog = new JFileChooser();
            dialog.setFileFilter(new FileNameExtensionFilter("txt", "txt"));
            dialog.showSaveDialog(this);

            try {
                _mapsCollection.SaveMap((String) Optional.ofNullable(ListBoxMaps.getSelectedValue()).orElse(""), dialog.getSelectedFile().getAbsolutePath());
                logger.info("Сохранение карты в файл: "+dialog.getSelectedFile().getAbsolutePath());
                JOptionPane.showMessageDialog(this, "Сохранение карты прошло успешно", "Итог", JOptionPane.INFORMATION_MESSAGE);
            } catch (Exception ex) {
                logger.error("Ошибка сохранения карты в файл: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Ошибка сохранения карты: "+ex.getMessage(), "Итог", JOptionPane.ERROR_MESSAGE);
            }
        });
        fileMenu.add(SaveMapMenuItem);

        JMenuItem LoadMapMenuItem = new JMenuItem("Загрузить карту");
        LoadMapMenuItem.addActionListener(e -> {
            JFileChooser dialog = new JFileChooser();
            dialog.setFileFilter(new FileNameExtensionFilter("txt", "txt"));
            dialog.showOpenDialog(this);

            try {
                _mapsCollection.LoadMap(dialog.getSelectedFile().getAbsolutePath());
                ReloadMaps();
                logger.info("Загрузка карты из файла: "+dialog.getSelectedFile().getAbsolutePath());
                JOptionPane.showMessageDialog(this, "Загрузка карты прошла успешно", "Итог", JOptionPane.INFORMATION_MESSAGE);
            } catch (Exception ex) {
                logger.error("Ошибка загрузки карты из файла: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Ошибка загрузки карты"+ex.getMessage(), "Итог", JOptionPane.ERROR_MESSAGE);
            }
        });
        fileMenu.add(LoadMapMenuItem);

        setJMenuBar(MenuBar);

        ListBoxMaps.addListSelectionListener(e -> {
            if(ListBoxMaps.getSelectedIndex()==-1)
                return;
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
            logger.info("Переход на карту "+ListBoxMaps.getSelectedValue());
        });

        DeleteMapButton.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() == -1)
            {
                return;
            }
            if(JOptionPane.showConfirmDialog(this,"Удалить карту "+ListBoxMaps.getSelectedValue().toString()+"?",
                    "Удаление",JOptionPane.YES_NO_OPTION)==0)
            {
                _mapsCollection.DelMap(ListBoxMaps.getSelectedValue().toString());
                logger.info("Карта "+ListBoxMaps.getSelectedValue()+" удалена");
                ReloadMaps();
            }
        });

        CreateMapButton.addActionListener(e -> {
            if (СomboBoxSelectorMap.getSelectedIndex() == -1 || TextFieldMap.getText()==null || TextFieldMap.getText().equals(""))
            {
                logger.warn("При добавлении карты не все данные были заполнены");
                JOptionPane.showMessageDialog(this,"Не все данные заполнены","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            if (!_mapsDict.containsKey(СomboBoxSelectorMap.getSelectedItem()))
            {
                logger.warn("Карты не существует");
                JOptionPane.showMessageDialog(this,"Нет такой карты","Ошибка",JOptionPane.ERROR_MESSAGE);
                return;
            }
            _mapsCollection.AddMap(TextFieldMap.getText(), _mapsDict.get(СomboBoxSelectorMap.getSelectedItem().toString()));
            ReloadMaps();
            logger.info("Добавлена карта: "+TextFieldMap.getText());
        });

        ButtonShowOnMap.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowOnMap();
            repaint();
        });

        ButtonShowStorage.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
            repaint();
        });

        ButtonAddWarship.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            FormWarshipConfig dialog=new FormWarshipConfig();
            dialog.addListener(obj -> {
                if (obj!=null) {
                    try {
                        DrawingObjectWarship warship = new DrawingObjectWarship(obj);

                        if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).plus(warship) >= 0) {
                            JOptionPane.showMessageDialog(this, "Объект добавлен", "Успех", JOptionPane.INFORMATION_MESSAGE);
                            bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                            repaint();
                            logger.info("Новый объект добавлен");
                        } else {
                            logger.warn("Не удалось добавить объект");
                            JOptionPane.showMessageDialog(this, "Не удалось добавить объект", "Ошибка", JOptionPane.ERROR_MESSAGE);
                        }
                    }catch (StorageOverflowException ex){
                        logger.error("Хранилище переполнено: "+ex.getMessage());
                        JOptionPane.showMessageDialog(this, "Ошибка. Хранилище переполнено: "+ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
                    }catch (Exception ex){
                        logger.fatal("Неизвестная ошибка: "+ex.getMessage());
                        JOptionPane.showMessageDialog(this, "Неизвестная ошибка: "+ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
                    }
                }
            });
            dialog.setVisible(true);
        });

        ButtonRemoveWarship.addActionListener(e -> {
            String txt=TextBoxPosition.getText();
            if (txt==null||ListBoxMaps.getSelectedIndex() ==-1)
            {
                return;
            }
            int result = JOptionPane.showConfirmDialog(
                    this,
                    "Удалить объект?",
                    "Удаление",
                    JOptionPane.YES_NO_CANCEL_OPTION);
            if (result!=0)
                return;
            int pos = Integer.parseInt(txt);

            try {
                if (_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).minus(pos) != null) {
                    JOptionPane.showMessageDialog(this, "Объект удален", "Успех", JOptionPane.INFORMATION_MESSAGE);
                    bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).ShowSet();
                    repaint();
                    logger.info("Объект на позиции"+pos+"удален");
                } else {
                    logger.warn("Не удалось удалить объект на позиции"+pos);
                    JOptionPane.showMessageDialog(this, "Не удалось удалить объект", "Ошибка", JOptionPane.ERROR_MESSAGE);
                }
            } catch(WarshipNotFoundException ex){
                logger.error("Ошибка удаления: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Ошибка удаления: "+ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
            }catch (Exception ex){
                logger.fatal("Неизвестная ошибка: "+ex.getMessage());
                JOptionPane.showMessageDialog(this, "Неизвестная ошибка: "+ex.getMessage(), "Ошибка", JOptionPane.ERROR_MESSAGE);
            }
        });

        ButtonShowDeleted.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex()==-1)
                return;
            DrawingObjectWarship warship=(DrawingObjectWarship)_mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).GetWarshipInDeleted();
            if(warship!=null){
                FormWarship dialog=new FormWarship();
                dialog.SetDrawingWarship(warship.GetWarship());
                dialog.setSize(1200,700);
                dialog.setModalityType(Dialog.ModalityType.APPLICATION_MODAL);
                dialog.setDefaultCloseOperation(JDialog.DISPOSE_ON_CLOSE);
                dialog.setVisible(true);
            }
        });

        ButtonUp.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Up);
                repaint();
            }
        });

        ButtonDown.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Down);
                repaint();
            }
        });

        ButtonRight.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Right);
                repaint();
            }
        });

        ButtonLeft.addActionListener(e -> {
            if (ListBoxMaps.getSelectedIndex() !=-1) {
                bufferedImage = _mapsCollection.get(ListBoxMaps.getSelectedValue().toString()).MoveObject(Direction.Left);
                repaint();
            }
        });
    }
}
